/*
 * 2007+ Copyright (c) Evgeniy Polyakov <johnpol@2ka.mipt.ru>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#define _ATFILE_SOURCE
#define _GNU_SOURCE
#define __USE_FILE_OFFSET64
#define __USE_LARGEFILE64

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/vfs.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>

#include <linux/types.h>

#include "fserver.h"

static int ac_count(int dirfd, char *path, struct ac_stat *st)
{
	int fd, err = 0;
	DIR *dir;
	struct dirent64 *d;
	struct stat stat;

	st->nr_dirs++;

	fd = openat(dirfd, path, O_RDONLY);
	if (fd == -1) {
		ulog_err("Failed to open '%s'", path);
		return -errno;
	}

	dir = fdopendir(fd);
	err = 0;

	while ((d = readdir64(dir)) != NULL) {
		if (d->d_name[0] == '.' && d->d_name[1] == '\0')
			continue;
		if (d->d_name[0] == '.' && d->d_name[1] == '.' && d->d_name[2] == '\0')
			continue;

		err = fstatat(fd, d->d_name, &stat, AT_SYMLINK_NOFOLLOW);
		if (!err) {
			st->used += stat.st_size;
		}

		if (d->d_type == DT_DIR) {
			err = ac_count(fd, d->d_name, st);
			if (err)
				break;
		} else {
			st->nr_files++;
		}
	}
	close(fd);

	return err;
}

int fserver_stat_root(int rootfd, char *path, struct ac_stat *st)
{
	struct statfs stfs;
	int err;

	memset(st, 0, sizeof(struct ac_stat));
	err = ac_count(rootfd, path, st);
	if (err)
		return err;

	err = statfs(path, &stfs);
	if (err)
		return err;

	st->avail = stfs.f_bavail * stfs.f_bsize;
	ulog("%s: files: %llu, dirs: %llu, used: %llu, avail: %llu.\n",
		path, st->nr_files, st->nr_dirs, st->used, st->avail);
	return 0;
}

